@@ -1,12 +1,14 @@
 import asyncio
 import logging
 import time
+from functools import lru_cache
 
 from asgiref.sync import async_to_sync
 from asgiref.sync import sync_to_async
 
 from opentelemetry_wrapper import instrument_decorate
 from opentelemetry_wrapper import instrument_logging
+from opentelemetry_wrapper import typecheck
 
 instrument_logging()
 
@@ -34,6 +36,9 @@ async def bitshift(x: int, shift_by: int):
 
 
 @instrument_decorate
+@lru_cache
+@instrument_decorate
+@typecheck
 async def multiply(multiplier: int, multiplicand: int):
     """
     very silly way to multiply things inspired by exponentiation-by-squaring
@@ -83,7 +88,6 @@ async def square(x):
 @sync_to_async
 @instrument_decorate
 @async_to_sync
-@instrument_decorate
 async def exponentiate(base, exponent):
     """
     exponentiation-by-squaring
